
/*  The Hacker's Diet  --  Database definitions  */

/*  Type definitions  */

typedef unsigned short weight;      /* Weight (integer with 2 fixed decimal places) */
typedef unsigned long trend;        /* Trend (long with 6 fixed decimal places) */
typedef unsigned char rung;         /* Exercise rung (8 bit integer, 255 = no entry) */

typedef unsigned int UInt;
typedef enum { false = 0, true = 1 } Boolean;

#define WEIGHT_MAX  0xFFFF
#define TREND_MAX   0xFFFFFFFF
#define RUNG_MAX    48
#define RUNG_ABSENT 255

#define WeightToTrend(w) ((w) * 10000L)
#define TrendToWeight(t) (((t) + 5000L) / 10000L)

#define wuKilogram  0
#define wuPound     1
#define wuStone     2

typedef enum { wKilogram = wuKilogram,
               wPound = wuPound,
               wStone = wuStone } weightunit;

#define CaloriesPerWeightUnit   ((prefs.weightUnit == wuKilogram) ? 7716 : 3500)
#define CaloriesPerMonthlyWeightUnit(m)   ((m->weightUnit == wuKilogram) ? 7716 : 3500)
#define DeltaTrendToCalories(t) ((TrendToWeight(t) * CaloriesPerWeightUnit) / 100L)

/*  Calories (gram) per Joule, and hence Calories (kilogram)
    per kiloJoule.  */

#define CaloriesPerJoule    0.239045

/*  Selected energy units per weight unit  */

#define EnergyPerMonthlyWeightUnit(m, joules) (CaloriesPerMonthlyWeightUnit(m) / \
                                               (joules ? CaloriesPerJoule : 1))         

#define min(x, y) (((x) < (y)) ? (x) : (y))
#define max(x, y) (((x) > (y)) ? (x) : (y))
#define abs(x) (((x) < 0) ? (-(x)) : (x))

#define timeOffset  2082844886LU      /* Time offset from Unix time() values */

/*  Monthly log record  */

#define CompressedComment   32        /* Compressed comment flag in day number */

struct hdMonthlyLog {
    unsigned short year;            /* Year for this log (watch not; not Y65K safe!) */
    unsigned char month;            /* Month (0 - 11) */
    unsigned char weightUnit;       /* Weight unit used in this record */
    trend trendCarryForward;        /* Trend carry-forward from last month */
    unsigned long lastModificationTime; /* Time of last modification */
    unsigned char flagsArray[4];    /* Bit array for maximum of 31 flags */
    rung rungArray[31];             /* Exercise rung array */
    weight weightArray[31];         /* Weight array */
    
    /*  Comments are stored as a running string, terminated by
        a zero byte.  Each comment consists of the following
        sequence of bytes:
        
                0:      Day of month (1 - 31) comment attached to
                1:      Length of comment string
                2-n:    Comment text
                
        The minimum comment array length is one byte, the zero
        terminator.
        
        Duplicate comment strings may (but need not be compressed).  A
        compressed comment string is always a two-byte sequence:
        
                0:      Day of month (1 - 31) comment attached to + CompressedComment
                1:      Previous day of month's (1 - 31) comment which is identical
    */
    
    char commentArray[1];           /* Comment array */
};

/*  Unpacked data for month  */

struct hdMonth {
    unsigned short year;            /* Year for this log (watch not; not Y65K safe!) */
    unsigned char month;            /* Month (0 - 11) */
    unsigned char weightUnit;       /* Weight unit used in this record */
    unsigned int days;              /* Days in this month */
    unsigned int firstDay;          /* Day of week of first day */
    trend trendCarryForward;        /* Trend carry-forward from last month */
    unsigned long lastModificationTime; /* Time of last modification */
    /*  Note that these arrays are indexed by day numbers 0-30, not 1-31 */
    rung rungArray[31];             /* Exercise rung array */
    weight weightArray[31];         /* Weight array */
    trend trendArray[31];           /* Running trend array */
    unsigned char flagsArray[31];   /* Flags */
    char *commentArray[31];         /* Pointers to comments */
};

/*  Function prototypes  */

extern int DaysInMonth(int month, int year);
extern struct hdMonth *unpackMonthRecord(struct hdMonthlyLog *md);
extern struct hdMonthlyLog *packMonthRecord(struct hdMonth *m, UInt *ml);
extern struct hdMonth *newMonth(int year, int month);
extern void disposeMonth(struct hdMonth *m);
